package au.asn.ucc.ticker.impl;

import au.asn.ucc.ticker.PriorityEnum;
import au.asn.ucc.ticker.Ticker;
import java.util.Date;

/**
 * A very simple implementation of Ticker that prepares an unanimated Image for
 * rendering, then waits for a period of time before requesting deactivation.
 * 
 * $Id: SimpleTicker.java 35 2009-12-29 20:07:36Z rvvs89 $
 * @author rvvs89
 */
public abstract class SimpleTicker extends Ticker {

    private Date end;
    private final PriorityEnum priority;
    private final long timeout;

    protected SimpleTicker(final String name, final PriorityEnum priority, final long timeout) {
        super(name);
        end = null;
        this.timeout = timeout;
        this.priority = priority;
    }

    @Override
    protected final void backgroundLoop() {
        setPriority(priority);
    }

    @Override
    protected final void updateLoop() throws InterruptedException {
        try {
            synchronized (this) {
                end = new Date();
                end.setTime(end.getTime() + timeout);
            }
            invokeRender();
            synchronized (this) {
                wait();
            }
        } finally {
            synchronized (this) {
                end = null;
            }
        }
    }

    @Override
    public final boolean canDeactivate() {
        return true;
    }

    @Override
    public final synchronized boolean shouldDeactivate() {
        return end == null ? true : new Date().after(end);
    }
}
